@@ -47,7 +47,7 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
   : QObject(parent)
   , isConnected_(true)
   , userSettings_{userSettings}
-  , notificationsManager(this)
+  , notificationsManager(new NotificationsManager(this))
   , callManager_(new CallManager(this))
 {
     setObjectName(QStringLiteral("chatPage"));
@@ -83,7 +83,7 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
     });
 
     connectivityTimer_.setInterval(CHECK_CONNECTIVITY_INTERVAL);
-    connect(&connectivityTimer_, &QTimer::timeout, this, [=]() {
+    connect(&connectivityTimer_, &QTimer::timeout, this, [this]() {
         if (http::client()->access_token().empty()) {
             connectivityTimer_.stop();
             return;
@@ -127,6 +127,12 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
           }
       });
 
+    connect(this,
+            &ChatPage::internalKnock,
+            this,
+            qOverload<const QString &, const std::vector<std::string> &, QString, bool>(
+              &ChatPage::knockRoom),
+            Qt::QueuedConnection);
     connect(this, &ChatPage::leftRoom, this, &ChatPage::removeRoom);
     connect(this, &ChatPage::changeToRoom, this, &ChatPage::changeRoom, Qt::QueuedConnection);
     connect(this, &ChatPage::notificationsRetrieved, this, &ChatPage::sendNotifications);
@@ -141,21 +147,31 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
                 }
             });
 
-    connect(&notificationsManager,
+    connect(notificationsManager,
             &NotificationsManager::notificationClicked,
             this,
             [this](const QString &roomid, const QString &eventid) {
                 Q_UNUSED(eventid)
-                view_manager_->rooms()->setCurrentRoom(roomid);
-                MainWindow::instance()->requestActivate();
+                auto exWin = MainWindow::instance()->windowForRoom(roomid);
+                if (exWin) {
+                    exWin->requestActivate();
+                } else {
+                    view_manager_->rooms()->setCurrentRoom(roomid);
+                    MainWindow::instance()->requestActivate();
+                }
             });
-    connect(&notificationsManager,
+    connect(notificationsManager,
             &NotificationsManager::sendNotificationReply,
             this,
             [this](const QString &roomid, const QString &eventid, const QString &body) {
-                view_manager_->rooms()->setCurrentRoom(roomid);
                 view_manager_->queueReply(roomid, eventid, body);
-                MainWindow::instance()->requestActivate();
+                auto exWin = MainWindow::instance()->windowForRoom(roomid);
+                if (exWin) {
+                    exWin->requestActivate();
+                } else {
+                    view_manager_->rooms()->setCurrentRoom(roomid);
+                    MainWindow::instance()->requestActivate();
+                }
             });
 
     connect(
@@ -220,10 +236,21 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
 
     connect(this, &ChatPage::dropToLoginPageCb, this, &ChatPage::dropToLoginPage);
 
-    connectCallMessage<mtx::events::msg::CallInvite>();
-    connectCallMessage<mtx::events::msg::CallCandidates>();
-    connectCallMessage<mtx::events::msg::CallAnswer>();
-    connectCallMessage<mtx::events::msg::CallHangUp>();
+    connect(
+      this,
+      &ChatPage::startRemoveFallbackKeyTimer,
+      this,
+      [this]() {
+          QTimer::singleShot(std::chrono::minutes(5), this, &ChatPage::removeOldFallbackKey);
+          disconnect(
+            this, &ChatPage::newSyncResponse, this, &ChatPage::startRemoveFallbackKeyTimer);
+      },
+      Qt::QueuedConnection);
+
+    connectCallMessage<mtx::events::voip::CallInvite>();
+    connectCallMessage<mtx::events::voip::CallCandidates>();
+    connectCallMessage<mtx::events::voip::CallAnswer>();
+    connectCallMessage<mtx::events::voip::CallHangUp>();
 }
 
 void
@@ -270,6 +297,10 @@ ChatPage::deleteConfigs()
     settings->beginGroup(QStringLiteral("auth"));
     settings->remove(QLatin1String(""));
     settings->endGroup(); // auth
+    if (UserSettings::instance()->profile() != QLatin1String("")) {
+        settings->endGroup(); // profilename
+        settings->endGroup(); // profile
+    }
 
     http::client()->shutdown();
     cache::deleteData();
@@ -365,7 +396,7 @@ ChatPage::bootstrap(QString userid, QString homeserver, QString token)
 
         connect(cache::client(),
                 &Cache::removeNotification,
-                &notificationsManager,
+                notificationsManager,
                 &NotificationsManager::removeNotification);
 
     } catch (const lmdb::error &e) {
@@ -395,7 +426,7 @@ ChatPage::loadStateFromCache()
         nhlog::db()->critical("failed to restore cache: {}", e.what());
         emit dropToLoginPageCb(tr("Failed to restore save data. Please login again."));
         return;
-    } catch (const json::exception &e) {
+    } catch (const nlohmann::json::exception &e) {
         nhlog::db()->critical("failed to parse cache data: {}", e.what());
         emit dropToLoginPageCb(tr("Failed to restore save data. Please login again."));
         return;
@@ -416,6 +447,7 @@ ChatPage::loadStateFromCache()
     emit contentLoaded();
 
     // Start receiving events.
+    connect(this, &ChatPage::newSyncResponse, &ChatPage::startRemoveFallbackKeyTimer);
     emit trySyncCb();
 }
 
@@ -460,7 +492,7 @@ ChatPage::sendNotifications(const mtx::responses::Notifications &res)
                                             96,
                                             this,
                                             [this, item](QPixmap image) {
-                                                notificationsManager.postNotification(
+                                                notificationsManager->postNotification(
                                                   item, image.toImage());
                                             });
                 }
@@ -479,7 +511,7 @@ ChatPage::tryInitialSync()
 
     // Upload one time keys for the device.
     nhlog::crypto()->info("generating one time keys");
-    olm::client()->generate_one_time_keys(MAX_ONETIME_KEYS);
+    olm::client()->generate_one_time_keys(MAX_ONETIME_KEYS, true);
 
     http::client()->upload_keys(
       olm::client()->create_upload_keys_request(),
@@ -503,6 +535,7 @@ ChatPage::tryInitialSync()
               return;
           }
 
+          olm::client()->forget_old_fallback_key();
           olm::mark_keys_as_published();
 
           for (const auto &entry : res.one_time_key_counts)
@@ -554,25 +587,26 @@ ChatPage::startInitialSync()
             }
         }
 
-        nhlog::net()->info("initial sync completed");
+        QTimer::singleShot(0, this, [this, res] {
+            nhlog::net()->info("initial sync completed");
+            try {
+                cache::client()->saveState(res);
 
-        try {
-            cache::client()->saveState(res);
+                olm::handle_to_device_messages(res.to_device.events);
 
-            olm::handle_to_device_messages(res.to_device.events);
+                emit initializeViews(std::move(res));
+                emit initializeMentions(cache::getTimelineMentions());
 
-            emit initializeViews(std::move(res));
-            emit initializeMentions(cache::getTimelineMentions());
+                cache::calculateRoomReadStatus();
+            } catch (const lmdb::error &e) {
+                nhlog::db()->error("failed to save state after initial sync: {}", e.what());
+                startInitialSync();
+                return;
+            }
 
-            cache::calculateRoomReadStatus();
-        } catch (const lmdb::error &e) {
-            nhlog::db()->error("failed to save state after initial sync: {}", e.what());
-            startInitialSync();
-            return;
-        }
-
-        emit trySyncCb();
-        emit contentLoaded();
+            emit trySyncCb();
+            emit contentLoaded();
+        });
     });
 }
 
@@ -586,12 +620,13 @@ ChatPage::handleSyncResponse(const mtx::responses::Sync &res, const std::string
         }
     } catch (const lmdb::error &e) {
         nhlog::db()->warn("Logged out in the mean time, dropping sync");
+        return;
     }
 
     nhlog::net()->debug("sync completed: {}", res.next_batch);
 
     // Ensure that we have enough one-time keys available.
-    ensureOneTimeKeyCount(res.device_one_time_keys_count);
+    ensureOneTimeKeyCount(res.device_one_time_keys_count, res.device_unused_fallback_key_types);
 
     // TODO: fine grained error handling
     try {
@@ -659,16 +694,53 @@ ChatPage::trySync()
 }
 
 void
-ChatPage::joinRoom(const QString &room)
+ChatPage::knockRoom(const QString &room,
+                    const std::vector<std::string> &via,
+                    QString reason,
+                    bool failedJoin)
 {
     const auto room_id = room.toStdString();
-    joinRoomVia(room_id, {}, false);
+    bool confirmed     = false;
+    reason             = QInputDialog::getText(
+      nullptr,
+      tr("Knock on room"),
+      // clang-format off
+      failedJoin
+        ? tr("You failed to join %1. You can try to knock, so that others can invite you in. Do you want to do so?\nYou may optionally provide a reason for others to accept your knock:").arg(room)
+        : tr("Do you really want to knock on %1? You may optionally provide a reason for others to accept your knock:").arg(room),
+      // clang-format on
+      QLineEdit::Normal,
+      reason,
+      &confirmed);
+    if (!confirmed) {
+        return;
+    }
+
+    http::client()->knock_room(
+      room_id,
+      via,
+      [this, room_id](const mtx::responses::RoomId &, mtx::http::RequestErr err) {
+          if (err) {
+              emit showNotification(tr("Failed to knock room: %1")
+                                      .arg(QString::fromStdString(err->matrix_error.error)));
+              return;
+          }
+      },
+      reason.toStdString());
+}
+
+void
+ChatPage::joinRoom(const QString &room, const QString &reason)
+{
+    const auto room_id = room.toStdString();
+    joinRoomVia(room_id, {}, false, reason);
 }
 
 void
 ChatPage::joinRoomVia(const std::string &room_id,
                       const std::vector<std::string> &via,
-                      bool promptForConfirmation)
+                      bool promptForConfirmation,
+                      const QString &reason)
 {
     if (promptForConfirmation &&
         QMessageBox::Yes !=
@@ -679,15 +751,18 @@ ChatPage::joinRoomVia(const std::string &room_id,
         return;
 
     http::client()->join_room(
-      room_id, via, [this, room_id](const mtx::responses::RoomId &, mtx::http::RequestErr err) {
+      room_id,
+      via,
+      [this, room_id, reason, via](const mtx::responses::RoomId &, mtx::http::RequestErr err) {
           if (err) {
-              emit showNotification(
-                tr("Failed to join room: %1").arg(QString::fromStdString(err->matrix_error.error)));
+              if (err->matrix_error.errcode == mtx::errors::ErrorCode::M_FORBIDDEN)
+                  emit internalKnock(QString::fromStdString(room_id), via, reason, true);
+              else
+                  emit showNotification(tr("Failed to join room: %1")
+                                          .arg(QString::fromStdString(err->matrix_error.error)));
               return;
           }
 
-          emit tr("You joined the room");
-
           // We remove any invites with the same room_id.
           try {
               cache::removeInvite(room_id);
@@ -696,12 +771,20 @@ ChatPage::joinRoomVia(const std::string &room_id,
           }
 
           view_manager_->rooms()->setCurrentRoom(QString::fromStdString(room_id));
-      });
+      },
+      reason.toStdString());
 }
 
 void
 ChatPage::createRoom(const mtx::requests::CreateRoom &req)
 {
+    if (req.room_alias_name.find(":") != std::string::npos ||
+        req.room_alias_name.find("#") != std::string::npos) {
+        nhlog::net()->warn("Failed to create room: Some characters are not allowed in alias");
+        emit this->showNotification(tr("Room creation failed: Bad Alias"));
+        return;
+    }
+
     http::client()->create_room(
       req, [this](const mtx::responses::CreateRoom &res, mtx::http::RequestErr err) {
           if (err) {
@@ -723,7 +806,7 @@ ChatPage::createRoom(const mtx::requests::CreateRoom &req)
 }
 
 void
-ChatPage::leaveRoom(const QString &room_id)
+ChatPage::leaveRoom(const QString &room_id, const QString &reason)
 {
     http::client()->leave_room(
       room_id.toStdString(),
@@ -745,7 +828,8 @@ ChatPage::leaveRoom(const QString &room_id)
           }
 
           emit leftRoom(room_id);
-      });
+      },
+      reason.toStdString());
 }
 
 void
@@ -755,10 +839,8 @@ ChatPage::changeRoom(const QString &room_id)
 }
 
 void
-ChatPage::inviteUser(QString userid, QString reason)
+ChatPage::inviteUser(const QString &room, QString userid, QString reason)
 {
-    auto room = currentRoom();
-
     if (QMessageBox::question(nullptr,
                               tr("Confirm invite"),
                               tr("Do you really want to invite %1 (%2)?")
@@ -781,10 +863,8 @@ ChatPage::inviteUser(QString userid, QString reason)
       reason.trimmed().toStdString());
 }
 void
-ChatPage::kickUser(QString userid, QString reason)
+ChatPage::kickUser(const QString &room, QString userid, QString reason)
 {
-    auto room = currentRoom();
-
     bool confirmed;
     reason =
       QInputDialog::getText(nullptr,
@@ -812,10 +892,8 @@ ChatPage::kickUser(QString userid, QString reason)
       reason.trimmed().toStdString());
 }
 void
-ChatPage::banUser(QString userid, QString reason)
+ChatPage::banUser(const QString &room, QString userid, QString reason)
 {
-    auto room = currentRoom();
-
     bool confirmed;
     reason =
       QInputDialog::getText(nullptr,
@@ -843,10 +921,8 @@ ChatPage::banUser(QString userid, QString reason)
       reason.trimmed().toStdString());
 }
 void
-ChatPage::unbanUser(QString userid, QString reason)
+ChatPage::unbanUser(const QString &room, QString userid, QString reason)
 {
-    auto room = currentRoom();
-
     if (QMessageBox::question(nullptr,
                               tr("Confirm unban"),
                               tr("Do you really want to unban %1 (%2)?")
@@ -931,26 +1007,39 @@ ChatPage::verifyOneTimeKeyCountAfterStartup()
           nhlog::crypto()->info(
             "Fetched server key count {} {}", count, mtx::crypto::SIGNED_CURVE25519);
 
-          ensureOneTimeKeyCount(key_counts);
+          ensureOneTimeKeyCount(key_counts, std::nullopt);
       });
 }
 
 void
-ChatPage::ensureOneTimeKeyCount(const std::map<std::string, uint16_t> &counts)
+ChatPage::ensureOneTimeKeyCount(const std::map<std::string, uint16_t> &counts,
+                                const std::optional<std::vector<std::string>> &unused_fallback_keys)
 {
     if (auto count = counts.find(mtx::crypto::SIGNED_CURVE25519); count != counts.end()) {
+        bool replace_fallback_key = false;
+        if (unused_fallback_keys &&
+            std::find(unused_fallback_keys->begin(),
+                      unused_fallback_keys->end(),
+                      mtx::crypto::SIGNED_CURVE25519) == unused_fallback_keys->end())
+            replace_fallback_key = true;
         nhlog::crypto()->debug(
-          "Updated server key count {} {}", count->second, mtx::crypto::SIGNED_CURVE25519);
+          "Updated server key count {} {}, fallback keys supported: {}, new fallback key: {}",
+          count->second,
+          mtx::crypto::SIGNED_CURVE25519,
+          unused_fallback_keys.has_value(),
+          replace_fallback_key);
 
-        if (count->second < MAX_ONETIME_KEYS) {
-            const size_t nkeys = MAX_ONETIME_KEYS - count->second;
+        if (count->second < MAX_ONETIME_KEYS || replace_fallback_key) {
+            const size_t nkeys =
+              count->second < MAX_ONETIME_KEYS ? (MAX_ONETIME_KEYS - count->second) : 0;
 
             nhlog::crypto()->info("uploading {} {} keys", nkeys, mtx::crypto::SIGNED_CURVE25519);
-            olm::client()->generate_one_time_keys(nkeys);
+            olm::client()->generate_one_time_keys(nkeys, replace_fallback_key);
 
             http::client()->upload_keys(
               olm::client()->create_upload_keys_request(),
-              [](const mtx::responses::UploadKeys &, mtx::http::RequestErr err) {
+              [replace_fallback_key, this](const mtx::responses::UploadKeys &,
+                                           mtx::http::RequestErr err) {
                   if (err) {
                       nhlog::crypto()->warn("failed to update one-time keys: {}", err);
 
@@ -960,6 +1049,10 @@ ChatPage::ensureOneTimeKeyCount(const std::map<std::string, uint16_t> &counts)
 
                   // mark as published anyway, otherwise we may end up in a loop.
                   olm::mark_keys_as_published();
+
+                  if (replace_fallback_key) {
+                      emit startRemoveFallbackKeyTimer();
+                  }
               });
         } else if (count->second > 2 * MAX_ONETIME_KEYS) {
             nhlog::crypto()->warn("too many one-time keys, deleting 1");
@@ -977,6 +1070,13 @@ ChatPage::ensureOneTimeKeyCount(const std::map<std::string, uint16_t> &counts)
     }
 }
 
+void
+ChatPage::removeOldFallbackKey()
+{
+    olm::client()->forget_old_fallback_key();
+    olm::mark_keys_as_published();
+}
+
 void
 ChatPage::getProfileInfo()
 {
@@ -1099,8 +1199,8 @@ ChatPage::decryptDownloadedSecrets(mtx::secret_storage::AesHmacSha2KeyDescriptio
     stripped.remove(' ');
     stripped.remove('\n');
     stripped.remove('\t');
-    auto decryptionKey = mtx::crypto::key_from_recoverykey(stripped.toStdString(), keyDesc);
 
+    auto decryptionKey = mtx::crypto::key_from_recoverykey(stripped.toStdString(), keyDesc);
     if (!decryptionKey && keyDesc.passphrase) {
         try {
             decryptionKey = mtx::crypto::key_from_passphrase(text.toStdString(), keyDesc);
@@ -1124,6 +1224,8 @@ ChatPage::decryptDownloadedSecrets(mtx::secret_storage::AesHmacSha2KeyDescriptio
 
     for (const auto &[secretName, encryptedSecret] : secrets) {
         auto decrypted = mtx::crypto::decrypt(encryptedSecret, *decryptionKey, secretName);
+        nhlog::crypto()->debug("Secret {} decrypted: {}", secretName, !decrypted.empty());
+
         if (!decrypted.empty()) {
             cache::storeSecret(secretName, decrypted);
 
@@ -1136,7 +1238,7 @@ ChatPage::decryptDownloadedSecrets(mtx::secret_storage::AesHmacSha2KeyDescriptio
                       olm::client()->identity_keys().ed25519 &&
                     myKey.keys["curve25519:" + http::client()->device_id()] ==
                       olm::client()->identity_keys().curve25519) {
-                    json j = myKey;
+                    nlohmann::json j = myKey;
                     j.erase("signatures");
                     j.erase("unsigned");
 
@@ -1152,10 +1254,11 @@ ChatPage::decryptDownloadedSecrets(mtx::secret_storage::AesHmacSha2KeyDescriptio
 
                 if (deviceKeys->master_keys.user_id == http::client()->user_id().to_string() &&
                     deviceKeys->master_keys.keys["ed25519:" + mk.public_key()] == mk.public_key()) {
-                    json j = deviceKeys->master_keys;
+                    nlohmann::json j = deviceKeys->master_keys;
                     j.erase("signatures");
                     j.erase("unsigned");
-                    mtx::crypto::CrossSigningKeys master_key = j;
+                    mtx::crypto::CrossSigningKeys master_key =
+                      j.get<mtx::crypto::CrossSigningKeys>();
                     master_key.signatures[http::client()->user_id().to_string()]
                                          ["ed25519:" + http::client()->device_id()] =
                       olm::client()->sign_message(j.dump());
@@ -1166,7 +1269,8 @@ ChatPage::decryptDownloadedSecrets(mtx::secret_storage::AesHmacSha2KeyDescriptio
         }
     }
 
-    if (!req.signatures.empty())
+    if (!req.signatures.empty()) {
+        nhlog::crypto()->debug("Uploading new signatures: {}", nlohmann::json(req).dump(2));
         http::client()->keys_signatures_upload(
           req, [](const mtx::responses::KeySignaturesUpload &res, mtx::http::RequestErr err) {
               if (err) {
@@ -1184,16 +1288,18 @@ ChatPage::decryptDownloadedSecrets(mtx::secret_storage::AesHmacSha2KeyDescriptio
                                           mtx::errors::to_string(e.errcode),
                                           e.error);
           });
+    }
 }
 
 void
-ChatPage::startChat(QString userid)
+ChatPage::startChat(QString userid, std::optional<bool> encryptionEnabled)
 {
     auto joined_rooms = cache::joinedRooms();
     auto room_infos   = cache::getRoomInfo(joined_rooms);
 
     for (const std::string &room_id : joined_rooms) {
-        if (room_infos[QString::fromStdString(room_id)].member_count == 2) {
+        if (const auto &info = room_infos[QString::fromStdString(room_id)];
+            info.member_count == 2 && !info.is_space) {
             auto room_members = cache::roomMembers(room_id);
             if (std::find(room_members.begin(), room_members.end(), (userid).toStdString()) !=
                 room_members.end()) {
@@ -1211,8 +1317,16 @@ ChatPage::startChat(QString userid)
         return;
 
     mtx::requests::CreateRoom req;
-    req.preset     = mtx::requests::Preset::PrivateChat;
+    req.preset     = mtx::requests::Preset::TrustedPrivateChat;
     req.visibility = mtx::common::RoomVisibility::Private;
+
+    if (encryptionEnabled.value_or(false)) {
+        mtx::events::StrippedEvent<mtx::events::state::Encryption> enc;
+        enc.type              = mtx::events::EventType::RoomEncryption;
+        enc.content.algorithm = mtx::crypto::MEGOLM_ALGO;
+        req.initial_state.emplace_back(std::move(enc));
+    }
+
     if (utils::localUser() != userid) {
         req.invite    = {userid.toStdString()};
         req.is_direct = true;
@@ -1329,9 +1443,11 @@ ChatPage::handleMatrixUri(QString uri)
 
     if (sigil1 == u"u") {
         if (action.isEmpty()) {
-            auto t = view_manager_->rooms()->currentRoom();
-            if (t && cache::isRoomMember(mxid1.toStdString(), t->roomId().toStdString())) {
-                t->openUserProfile(mxid1);
+            auto t = MainWindow::instance()->focusedRoom();
+            if (!t.isEmpty() && cache::isRoomMember(mxid1.toStdString(), t.toStdString())) {
+                auto rm = view_manager_->rooms()->getRoomById(t);
+                if (rm)
+                    rm->openUserProfile(mxid1);
                 return true;
             }
             emit view_manager_->openGlobalUserProfile(mxid1);
@@ -1355,6 +1471,9 @@ ChatPage::handleMatrixUri(QString uri)
         if (action == u"join" || action.isEmpty()) {
             joinRoomVia(targetRoomId, vias);
             return true;
+        } else if (action == u"knock" || action.isEmpty()) {
+            knockRoom(mxid1, vias);
+            return true;
         }
         return false;
     } else if (sigil1 == u"r") {
@@ -1362,9 +1481,10 @@ ChatPage::handleMatrixUri(QString uri)
         auto targetRoomAlias = mxid1.toStdString();
 
         for (const auto &roomid : joined_rooms) {
-            auto aliases = cache::client()->getRoomAliases(roomid);
+            auto aliases =
+              cache::client()->getStateEvent<mtx::events::state::CanonicalAlias>(roomid);
             if (aliases) {
-                if (aliases->alias == targetRoomAlias) {
+                if (aliases->content.alias == targetRoomAlias) {
                     view_manager_->rooms()->setCurrentRoom(QString::fromStdString(roomid));
                     if (!mxid2.isEmpty())
                         view_manager_->showEvent(QString::fromStdString(roomid), mxid2);
@@ -1376,6 +1496,9 @@ ChatPage::handleMatrixUri(QString uri)
         if (action == u"join" || action.isEmpty()) {
             joinRoomVia(mxid1.toStdString(), vias);
             return true;
+        } else if (action == u"knock" || action.isEmpty()) {
+            knockRoom(mxid1, vias);
+            return true;
         }
         return false;
     }
@@ -1391,14 +1514,6 @@ ChatPage::handleMatrixUri(const QUrl &uri)
 bool
 ChatPage::isRoomActive(const QString &room_id)
 {
-    return MainWindow::instance()->isActive() && currentRoom() == room_id;
-}
-
-QString
-ChatPage::currentRoom() const
-{
-    if (view_manager_->rooms()->currentRoom())
-        return view_manager_->rooms()->currentRoom()->roomId();
-    else
-        return QString();
+    return QGuiApplication::focusWindow() && QGuiApplication::focusWindow()->isActive() &&
+           MainWindow::instance()->windowForRoom(room_id) == QGuiApplication::focusWindow();
 }
