@@ -17,6 +17,7 @@
 
 #include "Cache.h"
 #include "Config.h"
+#include "JdenticonProvider.h"
 #include "MainWindow.h"
 #include "MatrixClient.h"
 #include "UserSettingsPage.h"
@@ -87,9 +88,12 @@ UserSettings::load(std::optional<QString> profile)
     openImageExternal_ = settings.value(QStringLiteral("user/open_image_external"), false).toBool();
     openVideoExternal_ = settings.value(QStringLiteral("user/open_video_external"), false).toBool();
     decryptSidebar_    = settings.value(QStringLiteral("user/decrypt_sidebar"), true).toBool();
-    privacyScreen_     = settings.value(QStringLiteral("user/privacy_screen"), false).toBool();
+    spaceNotifications_ = settings.value(QStringLiteral("user/space_notifications"), true).toBool();
+    privacyScreen_      = settings.value(QStringLiteral("user/privacy_screen"), false).toBool();
     privacyScreenTimeout_ =
       settings.value(QStringLiteral("user/privacy_screen_timeout"), 0).toInt();
+    exposeDBusApi_ = settings.value(QStringLiteral("user/expose_dbus_api"), false).toBool();
+
     mobileMode_ = settings.value(QStringLiteral("user/mobile_mode"), false).toBool();
     emojiFont_  = settings.value(QStringLiteral("user/emoji_font_family"), "emoji").toString();
     baseFontSize_ =
@@ -129,7 +133,8 @@ UserSettings::load(std::optional<QString> profile)
     userId_        = settings.value(prefix + "auth/user_id", "").toString();
     deviceId_      = settings.value(prefix + "auth/device_id", "").toString();
     hiddenTags_    = settings.value(prefix + "user/hidden_tags", QStringList{}).toStringList();
-    hiddenPins_    = settings.value(prefix + "user/hidden_pins", QStringList{}).toStringList();
+    mutedTags_  = settings.value(prefix + "user/muted_tags", QStringList{"global"}).toStringList();
+    hiddenPins_ = settings.value(prefix + "user/hidden_pins", QStringList{}).toStringList();
     hiddenWidgets_ = settings.value(prefix + "user/hidden_widgets", QStringList{}).toStringList();
     recentReactions_ =
       settings.value(prefix + "user/recent_reactions", QStringList{}).toStringList();
@@ -150,6 +155,13 @@ UserSettings::load(std::optional<QString> profile)
 
     applyTheme();
 }
+
+bool
+UserSettings::useIdenticon() const
+{
+    return useIdenticon_ && JdenticonProvider::isAvailable();
+}
+
 void
 UserSettings::setMessageHoverHighlight(bool state)
 {
@@ -210,14 +222,21 @@ UserSettings::setGroupView(bool state)
 }
 
 void
-UserSettings::setHiddenTags(QStringList hiddenTags)
+UserSettings::setHiddenTags(const QStringList &hiddenTags)
 {
     hiddenTags_ = hiddenTags;
     save();
 }
 
 void
-UserSettings::setHiddenPins(QStringList hiddenTags)
+UserSettings::setMutedTags(const QStringList &mutedTags)
+{
+    mutedTags_ = mutedTags;
+    save();
+}
+
+void
+UserSettings::setHiddenPins(const QStringList &hiddenTags)
 {
     hiddenPins_ = hiddenTags;
     save();
@@ -225,7 +244,7 @@ UserSettings::setHiddenPins(QStringList hiddenTags)
 }
 
 void
-UserSettings::setHiddenWidgets(QStringList hiddenTags)
+UserSettings::setHiddenWidgets(const QStringList &hiddenTags)
 {
     hiddenWidgets_ = hiddenTags;
     save();
@@ -247,6 +266,17 @@ UserSettings::setCollapsedSpaces(QList<QStringList> spaces)
     save();
 }
 
+void
+UserSettings::setExposeDBusApi(bool state)
+{
+    if (exposeDBusApi_ == state)
+        return;
+
+    exposeDBusApi_ = state;
+    emit exposeDBusApiChanged(state);
+    save();
+}
+
 void
 UserSettings::setMarkdown(bool state)
 {
@@ -395,6 +425,16 @@ UserSettings::setDecryptSidebar(bool state)
     save();
 }
 
+void
+UserSettings::setSpaceNotifications(bool state)
+{
+    if (state == spaceNotifications_)
+        return;
+    spaceNotifications_ = state;
+    emit spaceNotificationsChanged(state);
+    save();
+}
+
 void
 UserSettings::setPrivacyScreen(bool state)
 {
@@ -527,6 +567,9 @@ UserSettings::setUseOnlineKeyBackup(bool useBackup)
     useOnlineKeyBackup_ = useBackup;
     emit useOnlineKeyBackupChanged(useBackup);
     save();
+
+    if (useBackup)
+        olm::download_full_keybackup();
 }
 
 void
@@ -753,6 +796,7 @@ UserSettings::save()
 
     settings.setValue(QStringLiteral("avatar_circles"), avatarCircles_);
     settings.setValue(QStringLiteral("decrypt_sidebar"), decryptSidebar_);
+    settings.setValue(QStringLiteral("space_notifications"), spaceNotifications_);
     settings.setValue(QStringLiteral("privacy_screen"), privacyScreen_);
     settings.setValue(QStringLiteral("privacy_screen_timeout"), privacyScreenTimeout_);
     settings.setValue(QStringLiteral("mobile_mode"), mobileMode_);
@@ -788,6 +832,7 @@ UserSettings::save()
     settings.setValue(QStringLiteral("use_identicon"), useIdenticon_);
     settings.setValue(QStringLiteral("open_image_external"), openImageExternal_);
     settings.setValue(QStringLiteral("open_video_external"), openVideoExternal_);
+    settings.setValue(QStringLiteral("expose_dbus_api"), exposeDBusApi_);
 
     settings.endGroup(); // user
 
@@ -805,6 +850,7 @@ UserSettings::save()
                       onlyShareKeysWithVerifiedUsers_);
     settings.setValue(prefix + "user/online_key_backup", useOnlineKeyBackup_);
     settings.setValue(prefix + "user/hidden_tags", hiddenTags_);
+    settings.setValue(prefix + "user/muted_tags", mutedTags_);
     settings.setValue(prefix + "user/hidden_pins", hiddenPins_);
     settings.setValue(prefix + "user/hidden_widgets", hiddenWidgets_);
     settings.setValue(prefix + "user/recent_reactions", recentReactions_);
@@ -898,6 +944,8 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
             return tr("Open videos with external program");
         case DecryptSidebar:
             return tr("Decrypt messages in sidebar");
+        case SpaceNotifications:
+            return tr("Show message counts for spaces");
         case PrivacyScreen:
             return tr("Privacy Screen");
         case PrivacyScreenTimeout:
@@ -972,6 +1020,8 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
             return tr("User signing key");
         case MasterKey:
             return tr("Master signing key");
+        case ExposeDBusApi:
+            return tr("Expose room information via D-Bus");
         }
     } else if (role == Value) {
         switch (index.row()) {
@@ -1026,6 +1076,8 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
             return i->openVideoExternal();
         case DecryptSidebar:
             return i->decryptSidebar();
+        case SpaceNotifications:
+            return i->spaceNotifications();
         case PrivacyScreen:
             return i->privacyScreen();
         case PrivacyScreenTimeout:
@@ -1091,6 +1143,8 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
               .has_value();
         case MasterKey:
             return cache::secret(mtx::secret_storage::secrets::cross_signing_master).has_value();
+        case ExposeDBusApi:
+            return i->exposeDBusApi();
         }
     } else if (role == Description) {
         switch (index.row()) {
@@ -1154,7 +1208,9 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
               "Show buttons to quickly reply, react or access additional options next to each "
               "message.");
         case ReadReceipts:
-            return tr("Show if your message was read.\nStatus is displayed next to timestamps.");
+            return tr(
+              "Show if your message was read.\nStatus is displayed next to timestamps.\nWarning: "
+              "If your homeserver does not support this, your rooms will never be marked as read!");
         case DesktopNotifications:
             return tr("Notify about received messages when the client is not currently focused.");
         case AlertOnNotification:
@@ -1177,6 +1233,9 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
         case DecryptSidebar:
             return tr("Decrypt the messages shown in the sidebar.\nOnly affects messages in "
                       "encrypted chats.");
+        case SpaceNotifications:
+            return tr(
+              "Choose where to show the total number of notifications contained within a space.");
         case PrivacyScreen:
             return tr("When the window loses focus, the timeline will\nbe blurred.");
         case MobileMode:
@@ -1235,6 +1294,12 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
               "Your most important key. You don't need to have it cached, since not caching "
               "it makes it less likely it can be stolen and it is only needed to rotate your "
               "other signing keys.");
+        case ExposeDBusApi:
+            return tr("Allow third-party plugins and applications to load information about rooms "
+                      "you are in via D-Bus. "
+                      "This can have useful applications, but it also could be used for nefarious "
+                      "purposes. Enable at your own risk.\n\n"
+                      "This setting will take effect upon restart.");
         }
     } else if (role == Type) {
         switch (index.row()) {
@@ -1279,6 +1344,8 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
         case OnlyShareKeysWithVerifiedUsers:
         case ShareKeysWithTrustedUsers:
         case UseOnlineKeyBackup:
+        case ExposeDBusApi:
+        case SpaceNotifications:
             return Toggle;
         case Profile:
         case UserId:
@@ -1371,7 +1438,7 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
             return fontDb.families();
         case EmojiFont:
             return fontDb.families(QFontDatabase::WritingSystem::Symbol);
-        case Ringtone:
+        case Ringtone: {
             QStringList l{
               QStringLiteral("Mute"),
               QStringLiteral("Default"),
@@ -1381,6 +1448,7 @@ UserSettingsModel::data(const QModelIndex &index, int role) const
                 l.push_back(i->ringtone());
             return l;
         }
+        }
     } else if (role == Good) {
         switch (index.row()) {
         case OnlineBackupKey:
@@ -1586,6 +1654,13 @@ UserSettingsModel::setData(const QModelIndex &index, const QVariant &value, int
                 return false;
         }
             return i->decryptSidebar();
+        case SpaceNotifications: {
+            if (value.userType() == QMetaType::Bool) {
+                i->setSpaceNotifications(value.toBool());
+                return true;
+            } else
+                return false;
+        }
         case PrivacyScreen: {
             if (value.userType() == QMetaType::Bool) {
                 i->setPrivacyScreen(value.toBool());
@@ -1711,6 +1786,13 @@ UserSettingsModel::setData(const QModelIndex &index, const QVariant &value, int
             } else
                 return false;
         }
+        case ExposeDBusApi: {
+            if (value.userType() == QMetaType::Bool) {
+                i->setExposeDBusApi(value.toBool());
+                return true;
+            } else
+                return false;
+        }
         }
     }
     return false;
@@ -1891,7 +1973,9 @@ UserSettingsModel::UserSettingsModel(QObject *p)
     connect(s.get(), &UserSettings::decryptSidebarChanged, this, [this]() {
         emit dataChanged(index(DecryptSidebar), index(DecryptSidebar), {Value});
     });
-
+    connect(s.get(), &UserSettings::spaceNotificationsChanged, this, [this] {
+        emit dataChanged(index(SpaceNotifications), index(SpaceNotifications), {Value});
+    });
     connect(s.get(), &UserSettings::trayChanged, this, [this]() {
         emit dataChanged(index(Tray), index(Tray), {Value});
         emit dataChanged(index(StartInTray), index(StartInTray), {Enabled});
@@ -1940,4 +2024,7 @@ UserSettingsModel::UserSettingsModel(QObject *p)
     connect(MainWindow::instance(), &MainWindow::secretsChanged, this, [this]() {
         emit dataChanged(index(OnlineBackupKey), index(MasterKey), {Value, Good});
     });
+    connect(s.get(), &UserSettings::exposeDBusApiChanged, this, [this] {
+        emit dataChanged(index(ExposeDBusApi), index(ExposeDBusApi), {Value});
+    });
 }
