@@ -1,4 +1,5 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: 2022 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -25,11 +26,14 @@ FallbackAuth::FallbackAuth(const QString &authType, const QString &session, QWid
 
     auto layout = new QVBoxLayout(this);
     layout->setSpacing(conf::modals::WIDGET_SPACING);
-    layout->setMargin(conf::modals::WIDGET_MARGIN);
+    layout->setContentsMargins(conf::modals::WIDGET_MARGIN,
+                               conf::modals::WIDGET_MARGIN,
+                               conf::modals::WIDGET_MARGIN,
+                               conf::modals::WIDGET_MARGIN);
 
     auto buttonLayout = new QHBoxLayout();
     buttonLayout->setSpacing(8);
-    buttonLayout->setMargin(0);
+    buttonLayout->setContentsMargins(0, 0, 0, 0);
 
     openBtn_    = new QPushButton(tr("Open Fallback in Browser"), this);
     cancelBtn_  = new QPushButton(tr("Cancel"), this);
@@ -56,8 +60,7 @@ FallbackAuth::FallbackAuth(const QString &authType, const QString &session, QWid
                                  "fallback/web?session=%3")
                            .arg(QString::fromStdString(http::client()->server()))
                            .arg(http::client()->port())
-                           .arg(session)
-                           .arg(authType);
+                           .arg(session, authType);
 
         QDesktopServices::openUrl(url);
     });
