@@ -15,11 +15,10 @@
 #include <QDateTime>
 #include <QTimer>
 #include <iostream>
+#include <tuple>
 
 static constexpr int TIMEOUT = 2 * 60 * 1000; // 2 minutes
 
-namespace msgs = mtx::events::msg;
-
 static mtx::events::msg::KeyVerificationMac
 key_verification_mac(mtx::crypto::SAS *sas,
                      mtx::identifiers::User sender,
@@ -39,6 +38,7 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
   , deviceIds(std::move(deviceIds_))
   , model_(model)
 {
+    nhlog::crypto()->debug("CREATING NEW FLOW, {}, {}", flow_type, (void *)this);
     if (deviceIds.size() == 1)
         deviceId = deviceIds.front();
 
@@ -139,7 +139,8 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
             &ChatPage::receivedDeviceVerificationCancel,
             this,
             [this](const mtx::events::msg::KeyVerificationCancel &msg) {
-                nhlog::crypto()->info("verification: received cancel");
+                nhlog::crypto()->info(
+                  "verification: received cancel, {} : {}", msg.code, msg.reason);
                 if (msg.transaction_id.has_value()) {
                     if (msg.transaction_id.value() != this->transaction_id)
                         return;
@@ -157,7 +158,8 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
       &ChatPage::receivedDeviceVerificationKey,
       this,
       [this](const mtx::events::msg::KeyVerificationKey &msg) {
-          nhlog::crypto()->info("verification: received key");
+          nhlog::crypto()->info(
+            "verification: received key, sender {}, state {}", sender, state().toStdString());
           if (msg.transaction_id.has_value()) {
               if (msg.transaction_id.value() != this->transaction_id)
                   return;
@@ -167,7 +169,7 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
           }
 
           if (sender) {
-              if (state_ != WaitingForOtherToAccept) {
+              if (state_ != WaitingForOtherToAccept && state_ != WaitingForKeys) {
                   this->cancelVerification(OutOfOrder);
                   return;
               }
@@ -197,8 +199,8 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
           if (this->sender == false) {
               this->sendVerificationKey();
           } else {
-              if (this->commitment != mtx::crypto::bin2base64_unpadded(mtx::crypto::sha256(
-                                        msg.key + this->canonical_json.dump()))) {
+              if (this->commitment != mtx::crypto::bin2base64_unpadded(
+                                        mtx::crypto::sha256(msg.key + this->canonical_json))) {
                   this->cancelVerification(DeviceVerificationFlow::Error::MismatchedCommitment);
                   return;
               }
@@ -265,10 +267,11 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
                   // verified it
                   for (const auto &mac : msg.mac) {
                       if (their_keys.master_keys.keys.count(mac.first)) {
-                          json j = their_keys.master_keys;
+                          nlohmann::json j = their_keys.master_keys;
                           j.erase("signatures");
                           j.erase("unsigned");
-                          mtx::crypto::CrossSigningKeys master_key = j;
+                          mtx::crypto::CrossSigningKeys master_key =
+                            j.get<mtx::crypto::CrossSigningKeys>();
                           master_key.signatures[utils::localUser().toStdString()]
                                                ["ed25519:" + http::client()->device_id()] =
                             olm::client()->sign_message(j.dump());
@@ -280,7 +283,7 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
                           auto device_id = this->deviceId.toStdString();
 
                           if (their_keys.device_keys.count(device_id)) {
-                              json j = their_keys.device_keys.at(device_id);
+                              nlohmann::json j = their_keys.device_keys.at(device_id);
                               j.erase("signatures");
                               j.erase("unsigned");
 
@@ -290,7 +293,7 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
                                   continue;
                               auto ssk = mtx::crypto::PkSigning::from_seed(*secret);
 
-                              mtx::crypto::DeviceKeys dev = j;
+                              mtx::crypto::DeviceKeys dev = j.get<mtx::crypto::DeviceKeys>();
                               dev.signatures[utils::localUser().toStdString()]
                                             ["ed25519:" + ssk.public_key()] = ssk.sign(j.dump());
 
@@ -302,7 +305,7 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
                   // Sign their master key with user signing key
                   for (const auto &mac : msg.mac) {
                       if (their_keys.master_keys.keys.count(mac.first)) {
-                          json j = their_keys.master_keys;
+                          nlohmann::json j = their_keys.master_keys;
                           j.erase("signatures");
                           j.erase("unsigned");
 
@@ -312,7 +315,8 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
                               continue;
                           auto usk = mtx::crypto::PkSigning::from_seed(*secret);
 
-                          mtx::crypto::CrossSigningKeys master_key = j;
+                          mtx::crypto::CrossSigningKeys master_key =
+                            j.get<mtx::crypto::CrossSigningKeys>();
                           master_key.signatures[utils::localUser().toStdString()]
                                                ["ed25519:" + usk.public_key()] = usk.sign(j.dump());
 
@@ -355,7 +359,7 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
       &ChatPage::receivedDeviceVerificationReady,
       this,
       [this](const mtx::events::msg::KeyVerificationReady &msg) {
-          nhlog::crypto()->info("verification: received ready");
+          nhlog::crypto()->info("verification: received ready {}", (void *)this);
           if (!sender) {
               if (msg.from_device != http::client()->device_id()) {
                   error_ = User;
@@ -403,7 +407,10 @@ DeviceVerificationFlow::DeviceVerificationFlow(QObject *,
               else {
                   this->deviceId = QString::fromStdString(msg.from_device);
               }
+          } else {
+              return;
           }
+          nhlog::crypto()->info("verification: received ready sending start {}", (void *)this);
           this->startVerificationRequest();
       });
 
@@ -473,7 +480,7 @@ DeviceVerificationFlow::next()
     } else {
         switch (state_) {
         case PromptStartVerification:
-            if (canonical_json.is_null())
+            if (canonical_json.empty())
                 sendVerificationReady();
             else // legacy path without request and ready
                 acceptVerificationRequest();
@@ -542,7 +549,10 @@ DeviceVerificationFlow::handleStartMessage(const mtx::events::msg::KeyVerificati
     } else if (msg.relations.references()) {
         if (msg.relations.references() != this->relation.event_id)
             return;
+    } else {
+        return;
     }
+
     if ((std::find(msg.key_agreement_protocols.begin(),
                    msg.key_agreement_protocols.end(),
                    "curve25519-hkdf-sha256") != msg.key_agreement_protocols.end()) &&
@@ -565,14 +575,26 @@ DeviceVerificationFlow::handleStartMessage(const mtx::events::msg::KeyVerificati
             return;
         }
         if (!sender)
-            this->canonical_json = nlohmann::json(msg);
+            this->canonical_json = nlohmann::json(msg).dump();
         else {
-            if (utils::localUser().toStdString() < this->toClient.to_string()) {
-                this->canonical_json = nlohmann::json(msg);
+            // resolve glare
+            if (std::tuple(this->toClient.to_string(), this->deviceId.toStdString()) <
+                std::tuple(utils::localUser().toStdString(), http::client()->device_id())) {
+                // treat this as if the user with the smaller mxid or smaller deviceid (if the mxid
+                // was the same) was the sender of "start"
+                this->canonical_json = nlohmann::json(msg).dump();
+                this->sender         = false;
+            }
+
+            if (msg.method != mtx::events::msg::VerificationMethods::SASv1) {
+                cancelVerification(DeviceVerificationFlow::Error::UnknownMethod);
+                return;
             }
         }
 
-        if (state_ != PromptStartVerification)
+        // If we didn't send "start", accept the verification (otherwise wait for the other side to
+        // accept
+        if (state_ != PromptStartVerification && !sender)
             this->acceptVerificationRequest();
     } else {
         this->cancelVerification(DeviceVerificationFlow::Error::UnknownMethod);
@@ -583,6 +605,10 @@ DeviceVerificationFlow::handleStartMessage(const mtx::events::msg::KeyVerificati
 void
 DeviceVerificationFlow::acceptVerificationRequest()
 {
+    if (acceptSent)
+        return;
+    acceptSent = true;
+
     mtx::events::msg::KeyVerificationAccept req;
 
     req.method                      = mtx::events::msg::VerificationMethods::SASv1;
@@ -594,7 +620,7 @@ DeviceVerificationFlow::acceptVerificationRequest()
     else if (this->method == mtx::events::msg::SASMethods::Decimal)
         req.short_authentication_string = {mtx::events::msg::SASMethods::Decimal};
     req.commitment = mtx::crypto::bin2base64_unpadded(
-      mtx::crypto::sha256(this->sas->public_key() + this->canonical_json.dump()));
+      mtx::crypto::sha256(this->sas->public_key() + this->canonical_json));
 
     send(req);
     setState(WaitingForKeys);
@@ -623,6 +649,10 @@ DeviceVerificationFlow::sendVerificationDone()
 void
 DeviceVerificationFlow::startVerificationRequest()
 {
+    if (startSent)
+        return;
+    startSent = true;
+
     mtx::events::msg::KeyVerificationStart req;
 
     req.from_device                  = http::client()->device_id();
@@ -636,12 +666,12 @@ DeviceVerificationFlow::startVerificationRequest()
     if (this->type == DeviceVerificationFlow::Type::ToDevice) {
         mtx::requests::ToDeviceMessages<mtx::events::msg::KeyVerificationStart> body;
         req.transaction_id   = this->transaction_id;
-        this->canonical_json = nlohmann::json(req);
+        this->canonical_json = nlohmann::json(req).dump();
     } else if (this->type == DeviceVerificationFlow::Type::RoomMsg && model_) {
         req.relations.relations.push_back(this->relation);
         // Set synthesized to surpress the nheko relation extensions
         req.relations.synthesized = true;
-        this->canonical_json      = nlohmann::json(req);
+        this->canonical_json      = nlohmann::json(req).dump();
     }
     send(req);
     setState(WaitingForOtherToAccept);
@@ -707,8 +737,8 @@ DeviceVerificationFlow::cancelVerification(DeviceVerificationFlow::Error error_c
     }
 
     this->error_ = error_code;
-    emit errorChanged();
     this->setState(Failed);
+    emit errorChanged();
 
     send(req);
 }
@@ -716,6 +746,10 @@ DeviceVerificationFlow::cancelVerification(DeviceVerificationFlow::Error error_c
 void
 DeviceVerificationFlow::sendVerificationKey()
 {
+    if (keySent)
+        return;
+    keySent = true;
+
     mtx::events::msg::KeyVerificationKey req;
 
     req.key = this->sas->public_key();
@@ -757,6 +791,10 @@ key_verification_mac(mtx::crypto::SAS *sas,
 void
 DeviceVerificationFlow::sendVerificationMac()
 {
+    if (macSent)
+        return;
+    macSent = true;
+
     std::map<std::string, std::string> key_list;
     key_list["ed25519:" + http::client()->device_id()] = olm::client()->identity_keys().ed25519;
 
