@@ -5,6 +5,10 @@
 
 #include "RoomlistModel.h"
 
+#include <QClipboard>
+#include <QGuiApplication>
+
+#include "Cache.h"
 #include "Cache_p.h"
 #include "ChatPage.h"
 #include "Logging.h"
@@ -14,6 +18,12 @@
 #include "TimelineModel.h"
 #include "TimelineViewManager.h"
 #include "UserSettingsPage.h"
+#include "Utils.h"
+#include "voip/CallManager.h"
+
+#ifdef NHEKO_DBUS_SYS
+#include <QDBusConnection>
+#endif
 
 RoomlistModel::RoomlistModel(TimelineViewManager *parent)
   : QAbstractListModel(parent)
@@ -117,7 +127,7 @@ RoomlistModel::data(const QModelIndex &index, int role) const
             case Roles::Time:
                 return room->lastMessage().descriptiveTime;
             case Roles::Timestamp:
-                return QVariant{static_cast<quint64>(room->lastMessage().timestamp)};
+                return QVariant{static_cast<quint64>(room->lastMessageTimestamp())};
             case Roles::HasUnreadMessages:
                 return this->roomReadStatus.count(roomid) && this->roomReadStatus.at(roomid);
             case Roles::HasLoudNotification:
@@ -288,6 +298,7 @@ RoomlistModel::addRoom(const QString &room_id, bool suppressInsertNotification)
                              {
                                Roles::HasLoudNotification,
                                Roles::LastMessage,
+                               Roles::Time,
                                Roles::Timestamp,
                                Roles::NotificationCount,
                                Qt::DisplayRole,
@@ -319,17 +330,20 @@ RoomlistModel::addRoom(const QString &room_id, bool suppressInsertNotification)
                                Qt::DisplayRole,
                              });
 
+            if (getRoomById(room_id)->isSpace())
+                return; // no need to update space notifications
+
             int total_unread_msgs = 0;
 
             for (const auto &room : qAsConst(models)) {
-                if (!room.isNull())
+                if (!room.isNull() && !room->isSpace())
                     total_unread_msgs += room->notificationCount();
             }
 
             emit totalUnreadMessageCountUpdated(total_unread_msgs);
         });
 
-        newRoom->updateLastMessage();
+        // newRoom->updateLastMessage();
 
         std::vector<QString> previewsToAdd;
         if (newRoom->isSpace()) {
@@ -604,6 +618,15 @@ RoomlistModel::initializeRooms()
     nhlog::db()->info("Restored {} rooms from cache", rowCount());
 
     endResetModel();
+
+#ifdef NHEKO_DBUS_SYS
+    if (MainWindow::instance()->dbusAvailable()) {
+        dbusInterface_ = new NhekoDBusBackend{this};
+        if (!QDBusConnection::sessionBus().registerObject(
+              "/", dbusInterface_, QDBusConnection::ExportScriptableSlots))
+            nhlog::ui()->warn("Failed to register rooms with D-Bus");
+    }
+#endif
 }
 
 void
@@ -619,15 +642,11 @@ RoomlistModel::clear()
 }
 
 void
-RoomlistModel::joinPreview(QString roomid, QString parentSpace)
+RoomlistModel::joinPreview(QString roomid)
 {
     if (previewedRooms.contains(roomid)) {
-        auto child = cache::client()->getStateEvent<mtx::events::state::space::Child>(
-          parentSpace.toStdString(), roomid.toStdString());
         ChatPage::instance()->joinRoomVia(
-          roomid.toStdString(),
-          (child && child->content.via) ? child->content.via.value() : std::vector<std::string>{},
-          false);
+          roomid.toStdString(), utils::roomVias(roomid.toStdString()), false);
     }
 }
 void
@@ -659,12 +678,12 @@ RoomlistModel::declineInvite(QString roomid)
             roomids.erase(roomids.begin() + idx);
             invites.remove(roomid);
             endRemoveRows();
-            ChatPage::instance()->leaveRoom(roomid);
+            ChatPage::instance()->leaveRoom(roomid, "");
         }
     }
 }
 void
-RoomlistModel::leave(QString roomid)
+RoomlistModel::leave(QString roomid, QString reason)
 {
     if (models.contains(roomid)) {
         auto idx = roomidToIndex(roomid);
@@ -674,11 +693,39 @@ RoomlistModel::leave(QString roomid)
             roomids.erase(roomids.begin() + idx);
             models.remove(roomid);
             endRemoveRows();
-            ChatPage::instance()->leaveRoom(roomid);
+            ChatPage::instance()->leaveRoom(roomid, reason);
         }
     }
 }
 
+RoomPreview
+RoomlistModel::getRoomPreviewById(QString roomid) const
+{
+    RoomPreview preview{};
+
+    if (invites.contains(roomid) || previewedRooms.contains(roomid)) {
+        std::optional<RoomInfo> i;
+        if (invites.contains(roomid)) {
+            i                 = invites.value(roomid);
+            preview.isInvite_ = true;
+        } else {
+            i                 = previewedRooms.value(roomid);
+            preview.isInvite_ = false;
+        }
+
+        if (i) {
+            preview.roomid_        = roomid;
+            preview.roomName_      = QString::fromStdString(i->name);
+            preview.roomTopic_     = QString::fromStdString(i->topic);
+            preview.roomAvatarUrl_ = QString::fromStdString(i->avatar_url);
+        } else {
+            preview.roomid_ = roomid;
+        }
+    }
+
+    return preview;
+}
+
 void
 RoomlistModel::setCurrentRoom(QString roomid)
 {
@@ -1037,6 +1084,14 @@ FilteredRoomlistModel::toggleTag(QString roomid, QString tag, bool on)
     }
 }
 
+void
+FilteredRoomlistModel::copyLink(QString roomid)
+{
+    auto link = QStringLiteral("%1?%2").arg(TimelineModel::getBareRoomLink(roomid),
+                                            TimelineModel::getRoomVias(roomid));
+    QGuiApplication::clipboard()->setText(link);
+}
+
 void
 FilteredRoomlistModel::nextRoomWithActivity()
 {
