@@ -9,6 +9,8 @@
 #include <QObject>
 #include <QPalette>
 
+#include "AliasEditModel.h"
+#include "PowerlevelsEditModels.h"
 #include "Theme.h"
 #include "UserProfile.h"
 
@@ -52,7 +54,16 @@ public:
     Q_INVOKABLE void setStatusMessage(QString msg) const;
     Q_INVOKABLE void showUserSettingsPage() const;
     Q_INVOKABLE void logout() const;
-    Q_INVOKABLE void openCreateRoomDialog() const;
+    Q_INVOKABLE void
+    createRoom(QString name, QString topic, QString aliasLocalpart, bool isEncrypted, int preset);
+    Q_INVOKABLE PowerlevelEditingModels *editPowerlevels(QString room_id_) const
+    {
+        return new PowerlevelEditingModels(room_id_);
+    }
+    Q_INVOKABLE AliasEditingModel *editAliases(QString room_id_) const
+    {
+        return new AliasEditingModel(room_id_.toStdString());
+    }
 
 public slots:
     void updateUserProfile();
@@ -63,7 +74,7 @@ signals:
 
     void openLogoutDialog();
     void openJoinRoomDialog();
-    void joinRoom(QString roomId);
+    void joinRoom(QString roomId, QString reason = "");
 
 private:
     QScopedPointer<UserProfile> currentUser_;
