@@ -8,12 +8,14 @@
 #include <QMimeDatabase>
 #include <QStandardPaths>
 
+#include "Cache.h"
 #include "Cache_p.h"
 #include "ChatPage.h"
 #include "Logging.h"
 #include "UserProfile.h"
 #include "Utils.h"
 #include "encryption/DeviceVerificationFlow.h"
+#include "encryption/VerificationManager.h"
 #include "mtx/responses/crypto.hpp"
 #include "timeline/TimelineModel.h"
 #include "timeline/TimelineViewManager.h"
@@ -187,7 +189,6 @@ UserProfile::fetchDeviceList(const QString &userID)
               nhlog::net()->warn("failed to query device keys: {},{}",
                                  mtx::errors::to_string(err->matrix_error.errcode),
                                  static_cast<int>(err->status_code));
-              return;
           }
 
           // Ensure local key cache is up to date
@@ -201,7 +202,6 @@ UserProfile::fetchDeviceList(const QString &userID)
                     nhlog::net()->warn("failed to query device keys: {},{}",
                                        mtx::errors::to_string(err->matrix_error.errcode),
                                        static_cast<int>(err->status_code));
-                    return;
                 }
 
                 emit verificationStatiChanged();
@@ -299,7 +299,7 @@ UserProfile::updateVerificationStatus()
 void
 UserProfile::banUser()
 {
-    ChatPage::instance()->banUser(this->userid_, QLatin1String(""));
+    ChatPage::instance()->banUser(roomid_, this->userid_, QLatin1String(""));
 }
 
 // void ignoreUser(){
@@ -309,13 +309,19 @@ UserProfile::banUser()
 void
 UserProfile::kickUser()
 {
-    ChatPage::instance()->kickUser(this->userid_, QLatin1String(""));
+    ChatPage::instance()->kickUser(roomid_, this->userid_, QLatin1String(""));
+}
+
+void
+UserProfile::startChat(bool encryption)
+{
+    ChatPage::instance()->startChat(this->userid_, encryption);
 }
 
 void
 UserProfile::startChat()
 {
-    ChatPage::instance()->startChat(this->userid_);
+    ChatPage::instance()->startChat(this->userid_, std::nullopt);
 }
 
 void
